@@ -34,27 +34,33 @@
 #include <libgnomeui/gnome-thumbnail.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include "typedefs.h"
+#include "file-data.h"
 
 #define SPECIAL_DIR(x) (! strcmp (x, "..") || ! strcmp (x, "."))
 #define errno_to_string() (gnome_vfs_result_to_string (gnome_vfs_result_from_errno ()))
 
+typedef void (*CopyDoneFunc) (const char *, GnomeVFSResult, gpointer);
 
 /* Async directory list */
 
 typedef struct _PathListData PathListData;
-typedef void (*PathListDoneFunc) (PathListData *dld, gpointer data);
+typedef gboolean (*PathListFilterFunc) (PathListData *pld, FileData *, gpointer data);
+typedef void (*PathListDoneFunc) (PathListData *pld, gpointer data);
 
 struct _PathListData {
-	GnomeVFSURI      *uri;
-	GnomeVFSResult    result;
-	GList            *files;               /* char* items. */
-	GList            *dirs;                /* char* items. */
-	PathListDoneFunc  done_func;
-	gpointer          done_data;
-	DoneFunc          interrupt_func;
-	gpointer          interrupt_data;
-	gboolean          interrupted;
-	GHashTable       *hidden_files;
+	GnomeVFSURI        *uri;
+	GnomeVFSResult      result;
+	GList              *files;               /* char* items. */
+	GList              *dirs;                /* char* items. */
+	PathListFilterFunc  filter_func;
+	gpointer            filter_data;
+	PathListDoneFunc    done_func;
+	gpointer            done_data;
+	DoneFunc            interrupt_func;
+	gpointer            interrupt_data;
+	gboolean            interrupted;
+	GHashTable         *hidden_files;
+	gboolean            fast_file_type;
 };
 
 typedef struct {
@@ -63,36 +69,42 @@ typedef struct {
 } PathListHandle;
 
 PathListData *      path_list_data_new            (void);
-void                path_list_data_free           (PathListData     *dli);
-void                path_list_handle_free         (PathListHandle   *handle);
-PathListHandle *    path_list_async_new           (const char       *uri,
-						   PathListDoneFunc  f,
-						   gpointer          data);
-void                path_list_async_interrupt     (PathListHandle   *handle,
-						   DoneFunc          f,
-						   gpointer          data);
-gboolean            path_list_new                 (const char       *path,
-						   GList           **files,
-						   GList           **dirs);
-GList *             path_list_dup                 (GList            *path_list);
-void                path_list_free                (GList            *list);
-void                path_list_print               (GList            *list);
-GList *             path_list_find_path           (GList            *list,
-						   const char       *path);
+void                path_list_data_free           (PathListData       *dli);
+void                path_list_handle_free         (PathListHandle     *handle);
+PathListHandle *    path_list_async_new           (const char         *uri,
+						   PathListFilterFunc  filter_func,
+		     				   gpointer            filter_data,
+		     				   gboolean            fast_file_type,
+						   PathListDoneFunc    done_func,
+						   gpointer            done_data);
+void                path_list_async_interrupt     (PathListHandle     *handle,
+						   DoneFunc            f,
+						   gpointer            data);
+gboolean            path_list_new                 (const char         *path,
+						   GList             **files,
+						   GList             **dirs);
+GList *             path_list_dup                 (GList              *path_list);
+void                path_list_free                (GList              *list);
+void                path_list_print               (GList              *list);
+GList *             path_list_find_path           (GList              *list,
+						   const char         *path);
 
 /* Directory utils */
 
 gboolean            dir_is_empty                  (const char       *s);
-gboolean            dir_make                      (const char       *directory,
+gboolean            dir_make                      (const char       *uri,
 						   mode_t            mode);
-gboolean            dir_remove                    (const char       *directory);
-gboolean            dir_remove_recursive          (const char       *directory);
+gboolean            dir_remove                    (const char       *uri);
+gboolean            dir_remove_recursive          (const char       *uri);
+gboolean            local_dir_remove_recursive    (const char       *path);
 
 gboolean            ensure_dir_exists             (const char       *a_path,
 						   mode_t            mode);
 GList *             dir_list_filter_and_sort      (GList            *dir_list,
 						   gboolean          names_only,
 						   gboolean          show_dot_files);
+gboolean            file_filter                   (FileData         *file,
+	     					   gboolean          show_hidden_files);
 
 typedef void (*VisitFunc) (char *real_file, char *rc_file, gpointer data);
 gboolean            visit_rc_directory_sync       (const char       *rc_dir,
@@ -120,9 +132,12 @@ gboolean            file_copy                     (const char       *from,
 						   const char       *to);
 gboolean            file_move                     (const char       *from,
 						   const char       *to);
+gboolean            local_file_move               (const char       *from,
+						   const char       *to);
 GnomeVFSResult      file_rename                   (const char       *old_path,
 						   const char       *new_path);
 gboolean            file_unlink                   (const char       *path);
+void		    delete_thumbnail	          (const char       *path);
 gboolean            mime_type_is                  (const char       *mime_type,
 	      				 	   const char       *value);
 gboolean            image_is_type                 (const char       *name,
@@ -156,7 +171,10 @@ gboolean            uri_has_scheme                (const char       *uri);
 gboolean            uri_scheme_is_file            (const char       *uri);
 gboolean            uri_scheme_is_catalog         (const char       *uri);
 gboolean            uri_scheme_is_search          (const char       *uri);
-char *              get_uri_from_path             (const char       *path);
+char *              add_scheme_if_absent          (const char       *path);
+char *              add_filename_to_uri           (const char       *uri,
+						   const char       *filename);
+char *              get_uri_from_local_path       (const char       *path);
 char *              get_uri_display_name          (const char       *uri);
 G_CONST_RETURN char*file_name_from_path           (const char       *path);
 char *              get_local_path_from_uri       (const char       *uri);
@@ -168,8 +186,8 @@ gboolean            same_uri                      (const char       *uri1,
 						   const char       *uri2);
 char               *basename_for_display          (const char       *uri);
 
-char *              get_path_relative_to_dir      (const char       *filename,
-						   const char       *destdir);
+char *              get_path_relative_to_uri      (const char       *uri,
+						   const char       *desturi);
 char *              remove_level_from_path        (const char       *path);
 
 char *              remove_ending_separator       (const char       *path);
@@ -177,6 +195,9 @@ char *              remove_special_dirs_from_path (const char       *path);
 
 GnomeVFSURI *       new_uri_from_path             (const char       *path);
 char *              new_path_from_uri             (GnomeVFSURI      *uri);
+char *              build_uri                     (const char       *s1,
+						   const char       *s2,
+						   ...);
 GnomeVFSResult      resolve_all_symlinks          (const char       *text_uri,
 						   char            **resolved_text_uri);
 gboolean            uri_is_root                   (const char       *uri);
@@ -209,11 +230,16 @@ char *              remove_extension_from_path    (const char       *path);
 char *              get_temp_dir_name             (void);
 char *              get_temp_file_name            (const char       *tmpdir,
 						   const char       *ext);
-void		    remove_temp_file_and_dir      (char             *tmp_file);
-
 
 /* VFS extensions */
 
+typedef struct _CopyData CopyData;
+
+void                copy_data_cancel              (CopyData         *data);
+CopyData *          copy_file_async               (const char       *source_uri,
+		 				   const char       *target_uri,
+		 				   CopyDoneFunc      done_func,
+		 				   gpointer          done_data);
 GnomeVFSResult      _gnome_vfs_read_line          (GnomeVFSHandle   *handle,
 						   gpointer          buffer,
 						   GnomeVFSFileSize  buffer_size,
@@ -221,7 +247,7 @@ GnomeVFSResult      _gnome_vfs_read_line          (GnomeVFSHandle   *handle,
 GnomeVFSResult      _gnome_vfs_write_line         (GnomeVFSHandle   *handle,
 						   const char       *format,
 						   ...);
-GnomeVFSFileSize    get_dest_free_space           (const char       *path);
+GnomeVFSFileSize    get_destination_free_space    (const char       *path);
 const char *        get_mime_type                 (const char       *path);
 const char*         get_file_mime_type            (const char       *path,
 						   gboolean          fast_file_type);
@@ -230,26 +256,34 @@ gboolean            is_mime_type_writable         (const char       *mime_type);
 gboolean            check_permissions             (const char       *path,
 						   int               mode);
 gboolean	    is_local_file                 (const char       *filename);
-void	            prune_cache			  (void);
+char *              get_cache_filename_from_uri   (const char       *uri);
+char *              get_cache_uri_from_uri        (const char       *uri);
+void                free_cache                    (void);
+void                check_cache_free_space        (void);
+CopyData *          copy_remote_file_to_cache     (FileData         *file,
+						   CopyDoneFunc      done_func,
+						   gpointer          done_data);
+CopyData *          update_file_from_cache        (FileData         *file,
+						   CopyDoneFunc      done_func,
+						   gpointer          done_data);
 char* 		    obtain_local_file             (const char       *remote_filename);
 gboolean	    copy_cache_file_to_remote_uri (const char       *local_filename,
                                                    const char       *dest_uri);
 GHashTable *        read_dot_hidden_file          (const char       *uri);
 
 /* Pixbuf + VFS */
-GdkPixbuf*	    gth_pixbuf_new_from_uri	      (const char            *filename,
-						       GError 	            **error,
-				                       gint                   requested_width_if_used,
-				                       gint                   requested_height_if_used,
-				                       const char            *mime_type);
 
-GdkPixbufAnimation* gth_pixbuf_animation_new_from_uri (const char            *filename,
-						       GError               **error,
-						       gint                   requested_width_if_used,
-						       gint	              requested_height_if_used,
-						       GnomeThumbnailFactory *factory,
-						       const char            *mime_type);
+GdkPixbuf*	    gth_pixbuf_new_from_file	       (FileData               *file,
+			  				GError                **error,
+			  				int                     requested_width,
+			  				int                     requested_height,
+			  				GnomeThumbnailFactory  *factory);
+GdkPixbufAnimation* gth_pixbuf_animation_new_from_file (FileData              *file,
+						        GError               **error,
+						        int                    requested_width,
+						        int	               requested_height,
+						        GnomeThumbnailFactory *factory);
 
 char *              xdg_user_dir_lookup               (const char            *type);
-
+					       
 #endif /* FILE_UTILS_H */
