@@ -33,6 +33,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 
 #include "file-utils.h"
+#include "file-data.h"
 #include "catalog.h"
 #include "comments.h"
 #include "dlg-search.h"
@@ -140,6 +141,7 @@ typedef struct {
 
 	GHashTable     *folders_comment;
 	GHashTable     *hidden_files;
+	GHashTable     *visited_dirs;
 } DialogData;
 
 
@@ -181,16 +183,21 @@ static void
 free_search_results_data (DialogData *data)
 {
 	if (data->files) {
-		path_list_free (data->files);
+		file_data_list_free (data->files);
 		data->files = NULL;
 	}
-
+	
 	if (data->dirs) {
 		path_list_free (data->dirs);
 		data->dirs = NULL;
 	}
-
+	
 	g_hash_table_foreach_remove (data->folders_comment, remove_folder_comment_cb, NULL);
+	
+	if (data->visited_dirs != NULL) {
+		g_hash_table_destroy (data->visited_dirs);
+		data->visited_dirs = NULL;
+	}
 }
 
 
@@ -261,8 +268,8 @@ search_clicked_cb (GtkWidget  *widget,
 	/* * start from */
 
 	full_path = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->s_start_from_filechooserbutton));
-
 	search_data_set_start_from (data->search_data, full_path);
+	gth_file_list_ignore_hidden_thumbs (data->file_list, ! is_local_file (full_path));
 	g_free (full_path);
 
 	/* * recursive */
@@ -319,6 +326,7 @@ static void
 destroy_progress_cb (GtkWidget *widget,
 		     DialogData *data)
 {
+	gth_file_list_stop (data->file_list);
 	cancel_progress_dlg_cb (widget, data);
 	gtk_widget_destroy (data->dialog);
 }
@@ -366,8 +374,10 @@ view_result_cb (GtkWidget  *widget,
 	catalog_set_path (catalog, catalog_path);
 	catalog_set_search_data (catalog, data->search_data);
 
-	for (scan = data->files; scan; scan = scan->next)
-		catalog_add_item (catalog, (gchar*) scan->data);
+	for (scan = data->files; scan; scan = scan->next) {
+		FileData *fd = scan->data;
+		catalog_add_item (catalog, fd->path);
+	}
 
 	if (! catalog_write_to_disk (catalog, &gerror))
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &gerror);
@@ -398,8 +408,10 @@ save_result_cb (GtkWidget  *widget,
 	catalog = catalog_new ();
 	catalog_set_path (catalog, catalog_path);
 	catalog_set_search_data (catalog, data->search_data);
-	for (scan = data->files; scan; scan = scan->next)
-		catalog_add_item (catalog, (gchar*) scan->data);
+	for (scan = data->files; scan; scan = scan->next) {
+		FileData *fd = scan->data;
+		catalog_add_item (catalog, fd->path);
+	}
 
 	if (! catalog_write_to_disk (catalog, &gerror))
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &gerror);
@@ -443,7 +455,6 @@ static void
 cancel_progress_dlg_cb (GtkWidget  *widget,
 			DialogData *data)
 {
-	gth_file_list_stop (data->file_list);
 	if (data->handle == NULL)
 		return;
 	gnome_vfs_async_cancel (data->handle);
@@ -545,6 +556,7 @@ dlg_search_ui (GthBrowser *browser,
 	data->catalog_path = catalog_path;
 	data->folders_comment = g_hash_table_new (g_str_hash, g_str_equal);
 	data->hidden_files = NULL;
+	data->visited_dirs = NULL;
 
 	/* Get the widgets. */
 
@@ -572,7 +584,8 @@ dlg_search_ui (GthBrowser *browser,
 		data->p_searching_in_hbox = glade_xml_get_widget (data->gui, "p_searching_in_hbox");
 		data->p_images_label = glade_xml_get_widget (data->gui, "p_images_label");
 		progress_hbox = glade_xml_get_widget (data->gui, "p_progress_hbox");
-	} else {
+	} 
+	else {
 		data->search_progress_dialog = glade_xml_get_widget (data->gui, "edit_search_progress_dialog");
 		data->p_current_dir_entry = glade_xml_get_widget (data->gui, "ep_current_dir_entry");
 		data->p_notebook = glade_xml_get_widget (data->gui, "ep_notebook");
@@ -586,16 +599,19 @@ dlg_search_ui (GthBrowser *browser,
 	}
 
 	data->file_list = gth_file_list_new ();
+	
 	gtk_widget_show_all (data->file_list->root_widget);
 	gtk_box_pack_start (GTK_BOX (progress_hbox), data->file_list->root_widget, TRUE, TRUE, 0);
 
 	/* Set widgets data. */
 
 	if (catalog_path == NULL) {
-		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->s_start_from_filechooserbutton), gth_browser_get_current_directory (data->browser));
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->s_include_subfold_checkbutton), eel_gconf_get_boolean (PREF_SEARCH_RECURSIVE, TRUE));
-
-	} else {
+		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (data->s_start_from_filechooserbutton),
+							 gth_browser_get_current_directory (data->browser));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->s_include_subfold_checkbutton),
+					      eel_gconf_get_boolean (PREF_SEARCH_RECURSIVE, TRUE));
+	} 
+	else {
 		Catalog    *catalog;
 		SearchData *search_data;
 
@@ -617,11 +633,13 @@ dlg_search_ui (GthBrowser *browser,
 
 		/**/
 
-		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->s_start_from_filechooserbutton), search_data->start_from);
+		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (data->s_start_from_filechooserbutton),
+						         search_data->start_from);
 
 		/**/
 
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->s_include_subfold_checkbutton), search_data->recursive);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->s_include_subfold_checkbutton), 
+					      search_data->recursive);
 
 		gtk_entry_set_text (GTK_ENTRY (data->s_filename_entry),
 				    search_data->file_pattern);
@@ -992,7 +1010,8 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 	for (node = list; node != NULL; node = node->next) {
 		GnomeVFSURI *full_uri = NULL;
 		char        *str_uri;
-		char	    *unesc_uri;
+		char        *real_uri;
+		char	    *unesc_uri;  
 
 		info = node->data;
 
@@ -1005,7 +1024,7 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 			unesc_uri = gnome_vfs_unescape_string (str_uri, "");
 
 			if (file_respects_search_criteria (data, unesc_uri))
-				files = g_list_prepend (files, str_uri);
+				files = g_list_prepend (files, file_data_new (str_uri, info));
 			else
 				g_free (str_uri);
 
@@ -1018,7 +1037,15 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 			if (g_hash_table_lookup (data->hidden_files, info->name) != NULL)
 				break;
 			full_uri = gnome_vfs_uri_append_path (data->uri, info->name);
-			data->dirs = g_list_prepend (data->dirs, gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE));
+			str_uri = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
+			if (resolve_all_symlinks (str_uri, &real_uri) == GNOME_VFS_OK) {
+				if (g_hash_table_lookup (data->visited_dirs, real_uri) == NULL) { 
+					data->dirs = g_list_prepend (data->dirs, g_strdup (real_uri));
+					g_hash_table_insert (data->visited_dirs, g_strdup (real_uri), GINT_TO_POINTER (1));
+				}
+				g_free (real_uri);
+			}
+			g_free (str_uri);
 			break;
 
 		default:
@@ -1032,9 +1059,19 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 	if (files != NULL)
 		add_file_list (data, files);
 
-	if (result == GNOME_VFS_ERROR_EOF) {
+	if (result != GNOME_VFS_OK) {
 		gboolean good_dir_to_search_into = TRUE;
 
+		if (result != GNOME_VFS_ERROR_EOF) {
+			char *path;
+
+			path = gnome_vfs_uri_to_string (data->uri,
+							GNOME_VFS_URI_HIDE_NONE);
+			/*g_warning ("Cannot load directory \"%s\": %s\n", path,
+				   gnome_vfs_result_to_string (result));*/
+			g_free (path);
+		}
+
 		if (! data->search_data->recursive) {
 			search_finished (data);
 			return;
@@ -1059,18 +1096,7 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 				search_dir_async (data, dir);
 			g_free (dir);
 		} while (! good_dir_to_search_into);
-
-	} else if (result != GNOME_VFS_OK) {
-		char *path;
-
-		path = gnome_vfs_uri_to_string (data->uri,
-						GNOME_VFS_URI_HIDE_NONE);
-		g_warning ("Cannot load directory \"%s\": %s\n", path,
-			   gnome_vfs_result_to_string (result));
-		g_free (path);
-
-		search_finished (data);
-	}
+	} 
 }
 
 
@@ -1078,18 +1104,30 @@ static void
 search_dir_async (DialogData *data,
 		  char       *dir)
 {
+	char *uri, *real_uri;
+	
 	_gtk_entry_set_filename_text (GTK_ENTRY (data->p_current_dir_entry), dir);
 
 	/**/
 
 	if (data->uri != NULL)
 		gnome_vfs_uri_unref (data->uri);
-	data->uri = new_uri_from_path (dir);
+	
+	uri = add_scheme_if_absent (dir);
+	if (! resolve_all_symlinks (uri, &real_uri) == GNOME_VFS_OK) {
+		g_free (uri);
+		return;
+	}
+	g_free (uri);
+		 
+	g_hash_table_insert (data->visited_dirs, g_strdup (real_uri), GINT_TO_POINTER (1));
+	data->uri = new_uri_from_path (real_uri);
+	g_free (real_uri);
 
 	if (data->hidden_files != NULL)
 		g_hash_table_destroy (data->hidden_files);
 	data->hidden_files = read_dot_hidden_file (dir);
-
+				      
 	gnome_vfs_async_load_directory_uri (
 		& (data->handle),
 		data->uri,
@@ -1116,8 +1154,11 @@ search_images_async (DialogData *data)
 	SearchData *search_data = data->search_data;
 
 	free_search_results_data (data);
+	data->visited_dirs = g_hash_table_new_full (g_str_hash,
+					      	    g_str_equal,
+					            (GDestroyNotify) g_free,
+					            NULL);
 	gth_file_list_set_list (data->file_list, NULL, pref_get_arrange_type (), pref_get_sort_order ());
-
 	search_dir_async (data, search_data->start_from);
 }
 
